import re
from typing import Optional, List, Dict
from pymysql.converters import escape_string
from .ListORM import ListORM
from .utils import where_simple
from .session import SQLSessionContract, SQLSession


class SQLStatement:
    _mode_map = {
        'insert': "INSERT INTO",
        'ignore': "INSERT IGNORE INTO",
        # 注意 REPLACE 会把没有传入的字段值置成默认置
        'replace': "REPLACE INTO",
    }

    @classmethod
    def insert(cls, table_name: str, data: dict, mode='insert'):
        fields = []
        values = []
        for k, v in data.items():
            fields.append(f"`{k}`")
            if v is None:
                values.append("NULL")
            else:
                v = escape_string(str(v)).replace("%", "%%")
                values.append(f"'{v}'")
        fields_str = ', '.join(fields)
        values_str = ', '.join(values)

        sql = [cls._mode_map[mode], cls._fmt_dot(table_name), f"({fields_str})", 'VALUES', f"({values_str})"]
        return ' '.join(sql)

    @classmethod
    def insert_all(cls, table_name: str, data: List[Dict], mode='replace'):
        fields_arr = data[0].keys()
        values_arr = []
        for row in data:
            values = []
            for key in fields_arr:
                v = row[key]
                if v is None:
                    values.append('NULL')
                else:
                    values.append("'{}'".format(escape_string(str(v)).replace("%", "%%")))
            values_arr.append(f"({','.join(values)})")
        fields_str = ','.join([f'`{key}`' for key in fields_arr])
        values_str = ','.join(values_arr)

        sql = [cls._mode_map[mode], cls._fmt_dot(table_name), f"({fields_str})", 'VALUES', values_str]
        return ' '.join(sql)

    @classmethod
    def update(cls, table_name, data, condition):
        set_clause = []
        for k, v in data.items():
            if v is None:
                set_clause.append(f"`{k}` = NULL")
            else:
                v = escape_string(str(v)).replace("%", "%%")
                set_clause.append(f"`{k}` = '{v}'")
        set_clause_str = ','.join(set_clause)

        sql = ['UPDATE', cls._fmt_dot(table_name), 'SET', set_clause_str, 'WHERE', condition.replace('?', ' %s ')]
        return ' '.join(sql)

    @classmethod
    def delete(cls, table_name, condition):
        sql = ['DELETE FROM', cls._fmt_dot(table_name), 'WHERE', condition.replace('?', ' %s ')]
        return ' '.join(sql)

    @classmethod
    def select(cls, table_name, condition, field='*', for_update=False):
        if type(table_name) is str:
            table_name = [table_name, []]
        main_table, join_arr = table_name

        table = cls._fmt_table(main_table)
        for item in join_arr:
            # table2,on,join = ['tabel2','','LEFT JOIN']
            if len(item) == 3:
                table2, on, join = item
            else:
                table2, on = item
                join = 'LEFT JOIN'
            table = f"{table} {join.upper()} {cls._fmt_table(table2)} ON {on}"

        # 书写顺序：SELECT -> FROM -> JOIN -> ON -> WHERE -> GROUP BY -> HAVING -> UNION -> ORDER BY -> LIMIT -> FOR UPDATE
        sql = ['SELECT', field, 'FROM', table, 'WHERE', condition.replace('?', ' %s '), ]
        if for_update:
            sql.append('FOR UPDATE')
        return ' '.join(sql)

    @classmethod
    def _fmt_dot(cls, dot_str: str):
        arr = dot_str.split('.')
        return '.'.join([f"`{i}`" for i in arr])

    @classmethod
    def _fmt_table(cls, table_str: str):
        # 去除首尾空格并规范化内部空格
        table_str = re.sub(r'\s+', ' ', table_str.strip())

        parts = table_str.split()
        if len(parts) == 2:
            # 格式: table alias
            return f"{cls._fmt_dot(parts[0])} {parts[1]}"
        elif len(parts) == 3 and parts[1].upper() == 'AS':
            # 格式: table as alias
            return f"{cls._fmt_dot(parts[0])} AS {parts[2]}"
        else:
            return table_str


class EaseMySQL:
    def __init__(self, db: Optional[SQLSessionContract] = None, **kwargs):
        self.db = db
        if self.db is None:
            self.db = SQLSession(**kwargs)

        self.last_sql = ''

    def one(self, table_name, condition, params=(), field='*', order=None, group=None, flat=False):
        condition = where_simple(condition)
        if group:
            condition += f' GROUP BY {group}'
        if order:
            condition += f' ORDER BY {order}'
        _sql = SQLStatement.select(table_name, condition, field)
        one = self.db.fetchone(_sql, params)
        self.last_sql = _sql
        if flat and one is not None:
            return list(one.values())[0]
        return one

    def all(self, table_name, condition='1', params=(), field='*', group=None, order=None, limit=None, list_orm=False):
        condition = where_simple(condition)
        if group:
            condition += f' GROUP BY {group}'
        if order:
            condition += f' ORDER BY {order}'
        if limit:
            condition += f' LIMIT {limit}'
        _sql = SQLStatement.select(table_name, condition, field)
        result = self.db.fetchall(_sql, params)
        self.last_sql = _sql
        return ListORM(result) if list_orm else result

    def delete(self, table_name, condition, params=()):
        _sql = SQLStatement.delete(table_name, where_simple(condition))
        _id, cnt = self.db.execute(_sql, params)
        self.last_sql = _sql
        return cnt

    def update(self, table_name, data, condition, params=()):
        _sql = SQLStatement.update(table_name, data, where_simple(condition))
        _id, cnt = self.db.execute(_sql, params)
        self.last_sql = _sql
        return cnt

    def update_many(self, table_name, data, condition, params=()):
        """
        UPDATE chapters
        SET chapter_number = %s,
            chapter_number_prob = %s,
            chapter_title = %s,
            chapter_title_prob = %s,
            processed = 1
        WHERE id = %s

        参数示例
        data:[{'a': 0, 'c': 0}, {'a': 1, 'c': 1},]
        params:[[10], [11],]
        """
        _set = []
        keys = data[0].keys()
        for field in keys:
            _set.append(f"`{field}` = %s")

        _params = []
        for idx, one in enumerate(data):
            row = []
            for field in keys:
                row.append(one[field])
            if len(params) > 0:
                row += params[idx]
            _params.append(tuple(row))

        _sql = ' '.join(['UPDATE', f'`{table_name}`', 'SET', ','.join(_set), 'WHERE', condition])
        _id, cnt = self.db.executemany(_sql, _params)
        self.last_sql = _sql
        return cnt

    def insert(self, table_name, data, params=(), mode=None):
        if not data:
            return None
        if isinstance(data, dict):
            _sql = SQLStatement.insert(table_name, data, mode if mode else 'insert')
            _id, cnt = self.db.execute(_sql, params)
            self.last_sql = _sql
            return _id
        if isinstance(data, list):
            _sql = SQLStatement.insert_all(table_name, data, mode if mode else 'replace')
            _id, cnt = self.db.execute(_sql, params)
            self.last_sql = _sql
            return cnt
        return None

    def raw(self, sql, params=()):
        _id, cnt = self.db.execute(sql, params)
        self.last_sql = sql
        return _id, cnt

    def raw_one(self, sql, params=(), flat=False):
        one = self.db.fetchone(sql, params)
        self.last_sql = sql
        if flat and one is not None:
            return list(one.values())[0]
        return one

    def raw_all(self, sql, params=(), list_orm=False):
        result = self.db.fetchall(sql, params)
        self.last_sql = sql
        return ListORM(result) if list_orm else result

    def update_or_create(
            self,
            table_name,
            defaults: dict | None = None,
            create_defaults: dict | None = None,
            pk='id',
            **kwargs
    ) -> tuple:
        condition = where_simple(kwargs)
        _sql = SQLStatement.select(table_name, condition, for_update=True)
        instance = self.db.fetchone(_sql)
        self.last_sql = _sql
        if instance:
            if defaults:
                self.update(table_name, defaults, kwargs)
            created = False
        else:
            instance = {**(create_defaults or {}), **kwargs}
            instance[pk] = self.insert(table_name, instance)
            if not pk:
                instance = self.one(table_name, kwargs)
            created = True
        return instance, created


if __name__ == '__main__':
    # python -m src.easemysql.main
    # ValueError: unsupported format character 'A' (0x41) at index 3366
    print(SQLStatement.insert('test', dict(c1=1)))
    print(SQLStatement.insert_all('test', [dict(c1=1, c2=2), dict(c1=1, c2=2)], mode='insert'))
    print(SQLStatement.delete('test', 'id = 1'))
    print(SQLStatement.update('test', dict(c1=55, c2=66), 'id > 0'))
    print(SQLStatement.select('test', 'id > 0'))

    join_tables = [
        ["stat_book_info_show_pv_uv a", "m.zs_id = a.zs_id and a.app_id = 'com.sleepsounds.dztmmd'"],
        ["stat_book_read_progress b", "m.zs_id = b.zs_id", 'left join'],
    ]
    print(SQLStatement.select(['book m', join_tables], 'id > 0'))



import time
import threading
import pymysql
import warnings
from abc import ABC, abstractmethod
from dbutils.pooled_db import PooledDB

warnings.filterwarnings('ignore', category=pymysql.Warning)
lock = threading.Lock()


class SQLSessionContract(ABC):
    @abstractmethod
    def execute(self, sql, args=None):
        pass

    @abstractmethod
    def executemany(self, sql, args=None):
        pass

    @abstractmethod
    def fetchone(self, sql, args=None):
        pass

    @abstractmethod
    def fetchall(self, sql, args=None):
        pass


def singleton(cls):
    instances = {}

    def get_instance(*args, **kwargs):
        key = '@'.join(str(_) for _ in kwargs.values())
        with lock:
            if key not in instances:
                instances[key] = cls(*args, **kwargs)
        return instances[key]

    return get_instance


@singleton
class SQLSession(SQLSessionContract):
    # SQLSession(host='', user='', password='', database='')
    def __init__(self, **kwargs):
        conf = kwargs
        t1 = time.time()
        self.pool = PooledDB(
            creator=pymysql,
            maxconnections=conf.get('maxconnections', 10),  # 允许的最大连接数，超过时新请求将会等待连接被释放
            mincached=conf.get('mincached', 1),  # 启动时的初始连接数
            maxcached=conf.get('maxcached', 10),  # 最大空闲连接数量，超出的连接将会被关闭
            blocking=conf.get('blocking', True),  # 当连接数达到 maxconnections 并且没有空闲连接可用时，是否等待连接被释放。否则将抛出异常
            cursorclass=pymysql.cursors.DictCursor,  # 设置查询结果为字典
            # conv={
            #     pymysql.converters.FIELD_TYPE.DATETIME: str
            # },
            use_unicode=True,
            host=conf['host'],
            port=conf.get('port', 3306),
            user=conf['user'],
            password=conf['password'],
            database=conf['database']
        )
        self.log = ['{} 初始化连接，用时：{}'.format(time.time(), time.time() - t1)]
        self.execute_cnt = 0

    def __connection(self):
        # 如果你不调用 conn.close()，连接会保持开放状态，直到 PooledDB 的连接池自行处理（即回收或重用）这些连接。
        # 由于你使用了连接池，不调用 conn.close() 也不会导致资源泄漏，连接池会管理连接的生命周期。
        return self.pool.connection()

    def __cursor_execute(self, conn, cursor, sql, args):
        self.execute_cnt += 1
        try:
            cursor.execute(sql, args)
        except Exception as e:
            conn.rollback()
            # print(e)
            # print(sql)
            # print(args)
            # exit()
            raise Exception('{e} \n SLQ:{sql}; \n args:{args}'.format(e=e, sql=sql, args=args))

    def execute(self, sql, args=None):
        conn = self.__connection()
        with conn.cursor() as cursor:
            self.__cursor_execute(conn, cursor, sql, args)
            cnt = cursor.rowcount
            _id = cursor.lastrowid
        conn.commit()
        return _id, cnt

    def executemany(self, sql, args=None):
        conn = self.__connection()
        with conn.cursor() as cursor:
            self.execute_cnt += 1
            cursor.executemany(sql, args)
            cnt = cursor.rowcount
            _id = cursor.lastrowid
        conn.commit()
        return _id, cnt

    def fetchone(self, sql, args=None):
        conn = self.__connection()
        with conn.cursor() as cursor:
            self.__cursor_execute(conn, cursor, sql, args)
            result = cursor.fetchone()
        return result

    def fetchall(self, sql, args=None):
        conn = self.__connection()
        with conn.cursor() as cursor:
            self.__cursor_execute(conn, cursor, sql, args)
            result = cursor.fetchall()
        return result



class LookupResolve:
    def __init__(self, lookup):
        self.lookup = lookup

    def match(self, value, query):
        lookup = self.lookup.lower()
        if lookup == 'in':
            return value in query
        if lookup == 'contains':
            return query in f"{value}"
        return True

    @classmethod
    def and_match(cls, item: dict, **kwargs):
        match = True
        for key, value in kwargs.items():
            if '__' in key:
                key, lookup = f"{key}".split('__')
                if not cls(lookup).match(item.get(key), value):
                    match = False
                    break
            else:
                if item.get(key) != value:
                    match = False
                    break
        return match

    @classmethod
    def or_match(cls, item: dict, *args):
        if not args:
            return True
        match = []
        for where_dic in args:
            match.append(cls.and_match(item, **where_dic))
        return any(match)


class ListORM:
    def __init__(self, data_list: list):
        self._data = data_list

    def __iter__(self):
        for item in self._data:
            yield item

    def __len__(self):
        return len(self._data)

    def __repr__(self):
        return f"<ListORM object at {id(self)}>===={str(self._data)}===="

    def filter(self, *args, **kwargs):
        result = []
        for item in self._data:
            match = all([
                LookupResolve.or_match(item, *args),
                LookupResolve.and_match(item, **kwargs),
            ])
            if match:
                result.append(item)
        return ListORM(result)

    def group_by(self, *args, **kwargs):
        groups = dict()
        for item in self._data:
            key = tuple([item[field] for field in args])
            if key not in groups:
                groups[key] = []
            groups[key].append(item)

        result = []
        for keys, items in groups.items():
            _dic = dict(zip(args, keys))
            if field := kwargs.get('sum'):
                _dic[f"sum({field})"] = sum([i[field] for i in items])

            if field := kwargs.get('count'):
                _dic[f"count({field})"] = len(items)

            result.append(_dic)
        return ListORM(result)

    def order_by(self, *args):
        def _sort_key(row):
            arr = []
            for field in args:
                reverse = False
                if field.startswith('-'):
                    reverse = True  # reverse True 为倒序
                    field = field[1:]
                value = row[field]
                if isinstance(value, str) and len(value) > 0:
                    value = ord(value[0])
                if isinstance(value, (int, float)):
                    value = -value if reverse else value
                arr.append(value)

            return tuple(arr)

        self._data.sort(key=lambda x: _sort_key(x))
        return self

    def count(self):
        return len(self._data)

    def get(self) -> list:
        return self._data

    def get_map(self, *args):
        _map = dict()
        for item in self._data:
            if len(args) > 1:
                k = '@'.join([str(item[_]) for _ in args])
            else:
                k = item[args[0]]
            _map[k] = item
        return _map

    def first(self):
        if not self._data:
            return None
        return self._data[0]

    def sum(self, field):
        return sum([item[field] for item in self._data])

    def values_list(self, field, flat=True):
        arr = field.replace(' ', '').split(',')
        result = []
        for item in self._data:
            if flat:
                result.append(item[arr[0]])
            else:
                result.append([item[field] for field in arr])
        return result


if '__main__' == __name__:
    data = [
        {"hosp_id": 86, "p1": 'aa', "p2": 1, "p3": 3},
        {"hosp_id": 87, "p1": 'aa', "p2": 1, "p3": 8},
        {"hosp_id": 89, "p1": 'bb', "p2": 2, "p3": 5},
        {"hosp_id": 88, "p1": 'bb', "p2": 2, "p3": 1},
    ]

    orm = ListORM(data)
    # print(orm.filter(hosp_id=86).count())
    # print(orm.filter(hosp_id__in=[86, 87]).order_by('p3').get())
    # print(orm.filter(hosp_id__in=[86, 87]).sum('p3'))
    # print(orm.filter(hosp_id=86).first())
    # print(orm.filter().values_list('p1', flat=True))
    # for i in orm:
    #     print(i)
    #
    # print(orm.get_map('hosp_id'))
    # print(orm.values_list('hosp_id'))
    # print(len(orm))
    #
    # print('=' * 10)
    arr = orm.filter().order_by('p1', '-p3').get()
    for i in arr:
        print(i)

    print(orm.group_by('p2', sum='p3'))



from typing import Optional, List, Dict
import re


def is_safe_field_name(field_name: str) -> bool:
    if not field_name:
        return False

    if not re.match(r'^[a-zA-Z_][a-zA-Z0-9_.]*$', field_name):
        return False

    # 检查SQL关键字（部分常见的）
    sql_keywords = {
        'SELECT', 'INSERT', 'UPDATE', 'DELETE', 'DROP', 'CREATE',
        'ALTER', 'TRUNCATE', 'UNION', 'EXEC', 'EXECUTE'
    }
    if field_name.upper() in sql_keywords:
        return False

    return True


def _supported_sql_operators(condition: list) -> tuple[str, list]:
    operators = {
        'OR': 1,
        'AND': 1,
        'NOT': 1,
        'IN': 1,
        'NOT IN': 1,
        'BETWEEN': 1,
        '=': 1,
        '>': 1,
        '<': 1,
        '>=': 1,
        '<=': 1,
    }
    first = condition[0]
    if type(first) is str:
        first = first.upper()
        if first in operators:
            return first, condition[1:]
    return 'AND', condition[:]


def where_in(ids):
    if isinstance(ids, list):
        _str = ''
        for _id in ids:
            if isinstance(_id, (int, float)):
                _str += "{},".format(_id)
            else:
                _str += "'{}',".format(_id)
        return _str[:-1]
    return ids


def where_simple(condition) -> str:
    if isinstance(condition, str):
        return condition

    if isinstance(condition, list):
        return where_complex(condition)

    if isinstance(condition, dict):
        _arr = []
        for k, v in condition.items():
            if type(v) is list:
                _arr.append(f"{k} IN ({where_in(v)})")
            else:
                if isinstance(v, (int, float)):
                    _arr.append(f"{k} = {v}")
                else:
                    _arr.append(f"{k} = '{v}'")
        return ' AND '.join(_arr)

    raise Exception('Unsupported parameter type')


def where_complex(condition) -> str:
    """
    [
        'OR',
        [
            'AND',
            {'is_open': 1},
            'is_safe >= 10',
            ['>=', 'id', 100]
        ],
        ['IN', 'device_id', [1, 2, 3]]
    ]
    :return:
    """
    if not isinstance(condition, list):
        return '({})'.format(where_simple(condition))

    operator, values = _supported_sql_operators(condition)

    if operator in ['=', '>', '<', '>=', '<=']:
        return '({} {} {})'.format(values[0], operator, values[1])
    if operator in ['BETWEEN']:
        return '({} {} {} AND {})'.format(values[0], operator, values[1], values[2])
    if operator in ['IN', 'NOT IN']:
        return '({} {} ({}))'.format(values[0], operator, where_in(values[1]))
    if operator in ['NOT']:
        return '({} {})'.format(operator, where_complex(values))

    # or, and
    if operator == 'OR':
        return '({})'.format(f" {operator} ".join([where_complex(_) for _ in values]))
    return '{}'.format(f" {operator} ".join([where_complex(_) for _ in values]))


if '__main__' == __name__:
    # print(where_simple('id > 0'))
    # print(where_simple(dict(a=1, b=2, c=[1, 2, 3])))
    # print(where_simple(['BETWEEN', 'id', 1, 100]))
    # print(where_simple(['IN', 'id', [1, 100]]))
    # print(where_simple(['IN', 'id', ['1', '100']]))
    # print(where_simple(['in', 'id', '1,2,3']))
    print(where_simple([
        'id = 1',
        ['not', dict(id=1, name='a')],
        [
            dict(id=2),
            dict(id=3),
            ['or', dict(id=4), dict(id=5)]
        ]
    ]))
    print(where_simple([
        'OR',
        [
            {'is_open': 1},
            'is_safe >= 10',
            ['>=', 'id', 100],
            ['or', 'id = 1', 'id=2']
        ],
        ['IN', 'device_id', [1, 2, 3]],
    ]))



from .main import EaseMySQL, SQLStatement, SQLSession, SQLSessionContract
from .utils import where_in, where_simple, where_complex
from .ListORM import ListORM

"""
pip install pymysql --upgrade
pip install DBUtils --upgrade

注意1：
select * from test where nickname like "%风行水上%" 
不能直接执行这样的SQL，会提示：not enough arguments for format string
需要使用：cursor.execute('select * from test where nickname like %s', ["%风行水上%"])
原因是：
    pymysql 将字符串中的 % 视为需要格式化的占位符
    占位符：在 pymysql 中，参数化查询的占位符使用 %s，不论数据类型都统一写作 %s
    传入的参数以列表或字典形式提供给 cursor.execute() 方法

语义化版本 2.0.0
版本格式：主版本号.次版本号.修订号，版本号递增规则如下：
    主版本号：当你做了不兼容的 API 修改，
    次版本号：当你做了向下兼容的功能性新增，
    修订号：当你做了向下兼容的问题修正。
"""

VERSION = '1.2.2'

__all__ = [
    "VERSION",
    "EaseMySQL",
    "SQLStatement",
    "SQLSession",
    "SQLSessionContract",
    "where_in",
    "where_simple",
    "where_complex",
    "ListORM",
]



import sys
import unittest
from . import where_in, where_simple, EaseMySQL


class TestWhereIn(unittest.TestCase):
    """测试 where_in 函数"""

    def test_integer_list(self):
        """测试整数列表"""
        result = where_in([1, 2, 3])
        self.assertEqual(result, "1,2,3")

    def test_string_list(self):
        """测试字符串列表"""
        result = where_in(['a', 'b', 'c'])
        self.assertEqual(result, "'a','b','c'")

    def test_mixed_list(self):
        """测试混合类型列表"""
        result = where_in([1, 'a', 2.5])
        self.assertEqual(result, "1,'a',2.5")

    def test_single_value(self):
        """测试非列表值"""
        result = where_in(100)
        self.assertEqual(result, 100)

    def test_empty_list(self):
        """测试空列表 - 边界情况"""
        result = where_in([])
        self.assertEqual(result, "")


class TestWhereComplex(unittest.TestCase):
    def test_01(self):
        condition = [
            {'is_open': 1},
            'is_safe >= 10'
        ]
        a = where_simple(['AND', *condition])
        b = where_simple(condition)
        self.assertEqual(a, b)

    def test_02(self):
        condition = [
            'OR',
            [
                'AND',
                {'is_open': 1},
                'is_safe >= 10',
                ['>=', 'id', 100]
            ],
            ['IN', 'device_id', [1, 2, 3]]
        ]
        condition2 = [
            'OR',
            [
                {'is_open': 1},
                'is_safe >= 10',
                ['>=', 'id', 100]
            ],
            ['IN', 'device_id', [1, 2, 3]]
        ]
        result = where_simple(condition)
        self.assertEqual(
            '(((is_open = 1) AND (is_safe >= 10) AND (id >= 100)) OR (device_id IN (1,2,3)))',
            result
        )
        self.assertEqual(where_simple(condition2), result)


class TestEaseMySQL(unittest.TestCase):
    def test01(self):
        db = EaseMySQL(**dict(host='192.168.1.199', user='root', password='OzpWzB0fqoQN7jii', database='gateway_manage'))
        db.update_or_create('test', defaults={'name': 'a'}, id=1)
        db.update_or_create('test', create_defaults={'name': 'b'}, id=2)
        self.assertEqual(1, 1)



import setuptools

"""
打包教程：https://packaging.python.org/en/latest/tutorials/packaging-projects/

setuptools.setup 参数说明
name	包名称
version	包版本
author	程序的作者
author_email	程序的作者的邮箱地址
maintainer	维护者
maintainer_email	维护者的邮箱地址
url	程序的官网地址
license	程序的授权信息
description	程序的简单描述
long_description	程序的详细描述
platforms	程序适用的软件平台列表
classifiers	程序的所属分类列表
keywords	程序的关键字列表
packages	需要处理的包目录(通常为包含 __init__.py 的文件夹)
py_modules	需要打包的 Python 单文件列表
download_url	程序的下载地址
cmdclass	添加自定义命令
package_data	指定包内需要包含的数据文件
include_package_data	自动包含包内所有受版本控制(cvs/svn/git)的数据文件
exclude_package_data	当 include_package_data 为 True 时该选项用于排除部分文件
data_files	打包时需要打包的数据文件，如图片，配置文件等
ext_modules	指定扩展模块
scripts	指定可执行脚本,安装时脚本会被安装到系统 PATH 路径下
package_dir	指定哪些目录下的文件被映射到哪个源码包
entry_points	动态发现服务和插件，下面详细讲
python_requires	指定运行时需要的Python版本
requires	指定依赖的其他包
provides	指定可以为哪些模块提供依赖
install_requires	安装时需要安装的依赖包
extras_require	当前包的高级/额外特性需要依赖的分发包
tests_require	在测试时需要使用的依赖包
setup_requires	指定运行 setup.py 文件本身所依赖的包
dependency_links	指定依赖包的下载地址
zip_safe	不压缩包，而是以目录的形式安装
"""

with open('README.md', 'r', encoding='utf-8') as fp:
    long_description = fp.read()
with open('src/easemysql/__init__.py', 'r', encoding='utf-8') as fp:
    for line in fp.readlines():
        if line.startswith('VERSION'):
            version = line.split('=')[1].strip().strip("'").strip('"')
            break

setuptools.setup(
    version=version,
    name="easemysql",
    author="jaifire",
    author_email="boydearea@163.com",
    description="简单易用的 mysql 封装",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://gitee.com/jaifire/easemysql",

    # 将名为 "easemysql" 的包纳入分发,
    packages=["easemysql"],

    # packages 将从 src 目录加载，而不是从项目根目录
    # 空字符串 "" 键表示根包
    package_dir={"": "src"},

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
    ],
    python_requires=">=3.6"
)

"""
本地安装：python setup.py install

pip install build
pip install --upgrade twine

python -m build
twine upload dist/*
"""
